"""Tic-Tac-Toe is the simpelest of games.

The game is deterministic: there are no dice or random cards present.
The game is of complete information: every players knows the board.

This module serves as an example on how to implement simple games.
"""

from .Actions import TicTacToeAction
from .States import TicTacToeState
